@@ -27,6 +27,15 @@
 
 driver_t *drivers[N_DRIVERS] = { &maildir_driver, &imap_driver };
 
+int
+count_generic_messages( message_t *msgs )
+{
+	int count = 0;
+	for (; msgs; msgs = msgs->next)
+		count++;
+	return count;
+}
+
 void
 free_generic_messages( message_t *msgs )
 {
