@@ -287,7 +287,7 @@ send_imap_cmd( imap_store_t *ctx, imap_cmd_t *cmd )
 	int bufl, litplus, iovcnt = 1;
 	const char *buffmt;
 	conn_iovec_t iov[3];
-	char buf[1024];
+	char buf[4096];
 
 	cmd->tag = ++ctx->nexttag;
 	if (!cmd->param.data) {
@@ -444,7 +444,7 @@ imap_vprintf( const char *fmt, va_list ap )
 	char *d, *ed;
 	int maxlen;
 	char c;
-	char buf[1024]; /* Minimal supported command buffer size per IMAP spec. */
+	char buf[4096];
 
 	d = buf;
 	ed = d + sizeof(buf);
@@ -597,8 +597,9 @@ imap_refcounted_new_cmd( imap_cmd_refcounted_state_t *sts )
 		free( sts ); \
 	}
 
-#define DONE_REFCOUNTED_STATE_ARGS(sts, ...) \
+#define DONE_REFCOUNTED_STATE_ARGS(sts, finalize, ...) \
 	if (!--sts->gen.ref_count) { \
+		finalize \
 		sts->callback( sts->gen.ret_val, __VA_ARGS__, sts->callback_aux ); \
 		free( sts ); \
 	}
@@ -948,7 +949,7 @@ parse_date( const char *str )
 	struct tm datetime;
 
 	memset( &datetime, 0, sizeof(datetime) );
-	if (!(end = strptime( str, "%d-%b-%Y %H:%M:%S ", &datetime )))
+	if (!(end = strptime( str, "%e-%b-%Y %H:%M:%S ", &datetime )))
 		return -1;
 	if ((date = timegm( &datetime )) == -1)
 		return -1;
@@ -1102,7 +1103,6 @@ parse_fetch_rsp( imap_store_t *ctx, list_t *list, char *s ATTR_UNUSED )
 		if (status & M_FLAGS)
 			msgdata->flags = mask;
 	} else {
-		/* XXX this will need sorting for out-of-order (multiple queries) */
 		cur = nfcalloc( sizeof(*cur) );
 		*ctx->msgapp = &cur->gen;
 		ctx->msgapp = &cur->gen.next;
@@ -2073,7 +2073,7 @@ done_sasl_auth( imap_store_t *ctx, imap_cmd_t *cmd ATTR_UNUSED, int response )
 		int rc = sasl_client_step( ctx->sasl, NULL, 0, &interact, &out, &out_len );
 		if (process_sasl_step( ctx, rc, NULL, 0, interact, &out, &out_len ) < 0)
 			warn( "Warning: SASL reported failure despite successful IMAP authentication. Ignoring...\n" );
-		else if (out)
+		else if (out_len > 0)
 			warn( "Warning: SASL wants more steps despite successful IMAP authentication. Ignoring...\n" );
 	}
 
@@ -2174,7 +2174,7 @@ imap_open_store_authenticate2( imap_store_t *ctx )
 		free( enc );
 		return;
 	  notsasl:
-		if (!ctx->sasl || sasl_listmech( ctx->sasl, NULL, "", "", "", &saslavail, NULL, NULL ) != SASL_OK)
+		if (!ctx->sasl || sasl_listmech( ctx->sasl, NULL, "", " ", "", &saslavail, NULL, NULL ) != SASL_OK)
 			saslavail = "(none)";  /* EXTERNAL is always there anyway. */
 		if (!auth_login) {
 			error( "IMAP error: selected SASL mechanism(s) not available;\n"
@@ -2250,7 +2250,7 @@ imap_open_store_namespace( imap_store_t *ctx )
 
 	ctx->state = SST_HALF;
 	ctx->prefix = cfg->gen.path;
-	ctx->delimiter[0] = cfg->delimiter ? cfg->delimiter : 0;
+	ctx->delimiter[0] = cfg->delimiter;
 	if (((!ctx->prefix && cfg->use_namespace) || !cfg->delimiter) && CAP(NAMESPACE)) {
 		/* get NAMESPACE info */
 		if (!ctx->got_namespace)
@@ -2388,7 +2388,7 @@ imap_open_box_p2( imap_store_t *ctx, imap_cmd_t *gcmd, int response )
 	INIT_IMAP_CMD(imap_cmd_open_box_t, cmd, cmdp->callback, cmdp->callback_aux)
 	cmd->gen.param.lastuid = 1;
 	imap_exec( ctx, &cmd->gen, imap_open_box_p3,
-	           "UID FETCH *:* (UID)" );
+	           "UID FETCH * (UID)" );
 }
 
 static void
@@ -2587,6 +2587,47 @@ imap_load_box( store_t *gctx, uint minuid, uint maxuid, uint newuid, uint seenui
 	}
 }
 
+static int
+imap_sort_msgs_comp( const void *a_, const void *b_ )
+{
+	const message_t *a = *(const message_t * const *)a_;
+	const message_t *b = *(const message_t * const *)b_;
+
+	if (a->uid < b->uid)
+		return -1;
+	if (a->uid > b->uid)
+		return 1;
+	return 0;
+}
+
+static void
+imap_sort_msgs( imap_store_t *ctx )
+{
+	int count = count_generic_messages( ctx->msgs );
+	if (count <= 1)
+		return;
+
+	message_t **t = nfmalloc( sizeof(*t) * count );
+
+	message_t *m = ctx->msgs;
+	for (int i = 0; i < count; i++) {
+		t[i] = m;
+		m = m->next;
+	}
+
+	qsort( t, count, sizeof(*t), imap_sort_msgs_comp );
+
+	ctx->msgs = t[0];
+
+	int j;
+	for (j = 0; j < count - 1; j++)
+		t[j]->next = t[j + 1];
+	ctx->msgapp = &t[j]->next;
+	*ctx->msgapp = NULL;
+
+	free( t );
+}
+
 static void imap_submit_load_p2( imap_store_t *, imap_cmd_t *, int );
 
 static void
@@ -2615,7 +2656,10 @@ imap_submit_load_p2( imap_store_t *ctx, imap_cmd_t *cmd, int response )
 static void
 imap_submit_load_p3( imap_store_t *ctx, imap_load_box_state_t *sts )
 {
-	DONE_REFCOUNTED_STATE_ARGS(sts, ctx->msgs, ctx->total_msgs, ctx->recent_msgs)
+	DONE_REFCOUNTED_STATE_ARGS(sts, {
+		if (sts->gen.ret_val == DRV_OK)
+			imap_sort_msgs( ctx );
+	}, ctx->msgs, ctx->total_msgs, ctx->recent_msgs)
 }
 
 /******************* imap_fetch_msg *******************/
@@ -2922,7 +2966,7 @@ imap_find_new_msgs_p2( imap_store_t *ctx, imap_cmd_t *gcmd, int response )
 	cmd->uid = cmdp->uid;
 	cmd->gen.param.lastuid = 1;
 	imap_exec( ctx, &cmd->gen, imap_find_new_msgs_p3,
-	           "UID FETCH *:* (UID)" );
+	           "UID FETCH * (UID)" );
 }
 
 static void
@@ -3013,7 +3057,7 @@ imap_list_store_p2( imap_store_t *ctx, imap_cmd_t *cmd, int response )
 static void
 imap_list_store_p3( imap_store_t *ctx, imap_list_store_state_t *sts )
 {
-	DONE_REFCOUNTED_STATE_ARGS(sts, ctx->boxes)
+	DONE_REFCOUNTED_STATE_ARGS(sts, , ctx->boxes)
 }
 
 /******************* imap_cancel_cmds *******************/
@@ -3132,9 +3176,15 @@ imap_parse_store( conffile_t *cfg, store_conf_t **storep )
 			server->pass = nfstrdup( cfg->val );
 		else if (!strcasecmp( "PassCmd", cfg->cmd ))
 			server->pass_cmd = nfstrdup( cfg->val );
-		else if (!strcasecmp( "Port", cfg->cmd ))
-			server->sconf.port = parse_int( cfg );
-		else if (!strcasecmp( "Timeout", cfg->cmd ))
+		else if (!strcasecmp( "Port", cfg->cmd )) {
+			int port = parse_int( cfg );
+			if ((unsigned)port > 0xffff) {
+				error( "%s:%d: Invalid port number\n", cfg->file, cfg->line );
+				cfg->err = 1;
+			} else {
+				server->sconf.port = (ushort)port;
+			}
+		} else if (!strcasecmp( "Timeout", cfg->cmd ))
 			server->sconf.timeout = parse_int( cfg );
 		else if (!strcasecmp( "PipelineDepth", cfg->cmd )) {
 			if ((server->max_in_progress = parse_int( cfg )) < 1) {
