@@ -1153,28 +1153,29 @@ maildir_scan( maildir_store_t *ctx, msg_t_array_alloc_t *msglist )
 					goto retry;
 				}
 				int off, in_msgid = 0;
-				while ((want_tuid || want_msgid) && fgets( nbuf, sizeof(nbuf), f )) {
-					int bufl = strlen( nbuf );
-					if (bufl && nbuf[bufl - 1] == '\n')
+				char lnbuf[1000];  // Says RFC2822
+				while ((want_tuid || want_msgid) && fgets( lnbuf, sizeof(lnbuf), f )) {
+					int bufl = strlen( lnbuf );
+					if (bufl && lnbuf[bufl - 1] == '\n')
 						--bufl;
-					if (bufl && nbuf[bufl - 1] == '\r')
+					if (bufl && lnbuf[bufl - 1] == '\r')
 						--bufl;
 					if (!bufl)
 						break;
-					if (want_tuid && starts_with( nbuf, bufl, "X-TUID: ", 8 )) {
+					if (want_tuid && starts_with( lnbuf, bufl, "X-TUID: ", 8 )) {
 						if (bufl < 8 + TUIDL) {
 							error( "Maildir error: malformed X-TUID header (UID %u)\n", uid );
 							continue;
 						}
-						memcpy( entry->tuid, nbuf + 8, TUIDL );
+						memcpy( entry->tuid, lnbuf + 8, TUIDL );
 						want_tuid = 0;
 						in_msgid = 0;
 						continue;
 					}
-					if (want_msgid && starts_with_upper( nbuf, bufl, "MESSAGE-ID:", 11 )) {
+					if (want_msgid && starts_with_upper( lnbuf, bufl, "MESSAGE-ID:", 11 )) {
 						off = 11;
 					} else if (in_msgid) {
-						if (!isspace( nbuf[0] )) {
+						if (!isspace( lnbuf[0] )) {
 							in_msgid = 0;
 							continue;
 						}
@@ -1182,13 +1183,13 @@ maildir_scan( maildir_store_t *ctx, msg_t_array_alloc_t *msglist )
 					} else {
 						continue;
 					}
-					while (off < bufl && isspace( nbuf[off] ))
+					while (off < bufl && isspace( lnbuf[off] ))
 						off++;
 					if (off == bufl) {
 						in_msgid = 1;
 						continue;
 					}
-					entry->msgid = nfstrndup( nbuf + off, bufl - off );
+					entry->msgid = nfstrndup( lnbuf + off, bufl - off );
 					want_msgid = 0;
 					in_msgid = 0;
 				}
@@ -1569,7 +1570,7 @@ maildir_store_msg( store_t *gctx, msg_data_t *data, int to_trash,
 	uint uid;
 	char buf[_POSIX_PATH_MAX], nbuf[_POSIX_PATH_MAX], fbuf[NUM_FLAGS + 3], base[128];
 
-	bl = nfsnprintf( base, sizeof(base), "%ld.%d_%d.%s", (long)time( 0 ), Pid, ++MaildirCount, Hostname );
+	bl = nfsnprintf( base, sizeof(base), "%lld.%d_%d.%s", (long long)time( 0 ), Pid, ++MaildirCount, Hostname );
 	if (!to_trash) {
 #ifdef USE_DB
 		if (ctx->usedb) {
@@ -1750,8 +1751,8 @@ maildir_trash_msg( store_t *gctx, message_t *gmsg,
 	for (;;) {
 		nfsnprintf( buf, sizeof(buf), "%s/%s/%s", ctx->path, subdirs[gmsg->status & M_RECENT], msg->base );
 		s = strstr( msg->base, ((maildir_store_conf_t *)gctx->conf)->info_prefix );
-		nfsnprintf( nbuf, sizeof(nbuf), "%s/%s/%ld.%d_%d.%s%s", ctx->trash,
-		            subdirs[gmsg->status & M_RECENT], (long)time( 0 ), Pid, ++MaildirCount, Hostname, s ? s : "" );
+		nfsnprintf( nbuf, sizeof(nbuf), "%s/%s/%lld.%d_%d.%s%s", ctx->trash,
+		            subdirs[gmsg->status & M_RECENT], (long long)time( 0 ), Pid, ++MaildirCount, Hostname, s ? s : "" );
 		if (!rename( buf, nbuf ))
 			break;
 		if (!stat( buf, &st )) {
