@@ -30,9 +30,7 @@
 #endif
 
 #ifdef HAVE_LIBSSL
-typedef struct ssl_st SSL;
-typedef struct ssl_ctx_st SSL_CTX;
-typedef struct stack_st _STACK;
+# include <openssl/ssl.h>
 
 enum {
 	SSLv3 = 2,
@@ -45,7 +43,7 @@ enum {
 typedef struct {
 	char *tunnel;
 	char *host;
-	int port;
+	ushort port;
 	int timeout;
 #ifdef HAVE_LIBSSL
 	char *cert_file;
